provider "azurerm" {
  version = ">= 1.21.0"
}
 
variable "name" {
  description = "Name of the Kubernetes cluster"
  default     = "nevis-cluster"
}
 
variable "dns_prefix" {
  description = "A prefix that will be put in front of the auto-generated cluster ID: e.g., <prefix>-<number>.hcp.westeurope.azmk8s.io"
  default     = "nevis-cluster"
}
 
variable "vnet_subnet_cidr" {
  description = "Subnet in CIDR form where the AKS agents and Pods will be created. Must be a subnet of vnet_name and least a /24 network (recommended: /21)."
  default     = "10.1.0.0/21"
}
 
variable "service_cidr" {
  description = "The Network Range used by the Kubernetes service"
  default     = "10.100.0.0/16"
}
 
variable "dns_service_ip" {
  description = "IP address within the Kubernetes service address range that will be used by cluster service discovery (kube-dns)"
  default     = "10.100.0.10"
}
 
variable "docker_bridge_cidr" {
  description = "IP address (in CIDR notation) used as the Docker bridge IP address on nodes"
  default     = "172.17.0.1/16"
}
 
variable "admin_user" {
  default = "myownadmin"
}
 
variable "kubernetes_version" {
  description = "Kubernetes version, see: az aks get-versions --location 'West Europe -o table'"
  default     = "1.13.9"
}
 
variable "num_agents" {
  description = "Number of agents (should be at least 2 for productive use)"
  default     = "2"
}
 
variable "vm_size" {
  description = "VM type for the cluster nodes. Must have at least 1 Core and 2 GB of RAM."
  default     = "Standard_D2_v3"
}
 
variable "vm_disk_gb" {
  description = "Disk size of Node VM. Must be able to store all active Docker images"
  default     = "128"
}
 
variable "max_pods" {
  description = "The maximum number of pods that can run on each agent"
  default     = "30"
}
 
resource "azurerm_virtual_network" "vnet" {
  name                = "${var.vnet_name}"
  location            = "${var.location}"
  resource_group_name = "${var.resource_group_name}"
  address_space       = ["10.1.0.0/16"]
}
 
data "azurerm_client_config" "current" {}
 
data "azurerm_resource_group" "rgroup" {
  name = "${var.resource_group_name}"
}
 
resource "random_string" "ad_sp_password" {
  length  = 32
  special = false
}
 
resource "random_string" "ad_app_suffix" {
  length  = 8
  special = false
}
 
resource "azurerm_azuread_application" "ad_app" {
  name = "${data.azurerm_client_config.current.subscription_id}-${var.resource_group_name}-aks-${random_string.ad_app_suffix.result}"
}
 
resource "azurerm_azuread_service_principal" "ad_sp" {
  application_id = "${azurerm_azuread_application.ad_app.application_id}"
}
 
resource "azurerm_azuread_service_principal_password" "ad_sp_password" {
  service_principal_id = "${azurerm_azuread_service_principal.ad_sp.id}"
  value                = "${random_string.ad_sp_password.result}"
  end_date             = "2099-01-01T00:00:00Z"
}
 
resource "tls_private_key" "agents_admin_ssh_key" {
  algorithm = "RSA"
  rsa_bits  = 4096
}
 
resource "azurerm_subnet" "aks_subnet" {
  name                 = "${var.name}-subnet"
  resource_group_name  = "${var.resource_group_name}"
  virtual_network_name = "${azurerm_virtual_network.vnet.name}"
  address_prefix       = "${var.vnet_subnet_cidr}"
}
 
resource "azurerm_role_assignment" "aks_subnet_admin" {
  scope                = "/subscriptions/${data.azurerm_client_config.current.subscription_id}/resourceGroups/${var.resource_group_name}/providers/Microsoft.Network/virtualNetworks/${var.vnet_name}/subnets/${azurerm_subnet.aks_subnet.name}"
  role_definition_name = "Network Contributor"
  principal_id         = "${azurerm_azuread_service_principal.ad_sp.id}"
}
 
resource "azurerm_kubernetes_cluster" "aks_cluster" {
  name                = "${var.name}"
  location            = "${data.azurerm_resource_group.rgroup.location}"
  resource_group_name = "${var.resource_group_name}"
  depends_on          = ["azurerm_role_assignment.aks_subnet_admin"]
 
  dns_prefix         = "${var.dns_prefix}"
  kubernetes_version = "${var.kubernetes_version}"
 
  linux_profile {
    admin_username = "${var.admin_user}"
 
    ssh_key {
      key_data = "${tls_private_key.agents_admin_ssh_key.public_key_openssh}"
    }
  }
 
  agent_pool_profile {
    name            = "default"
    count           = "${var.num_agents}"
    vm_size         = "${var.vm_size}"
    os_type         = "Linux"
    os_disk_size_gb = "${var.vm_disk_gb}"
    vnet_subnet_id  = "${azurerm_subnet.aks_subnet.id}"
    max_pods        = "${var.max_pods}"
  }
 
  service_principal {
    client_id     = "${azurerm_azuread_application.ad_app.application_id}"
    client_secret = "${random_string.ad_sp_password.result}"
  }
 
  network_profile {
    network_plugin     = "azure"
    service_cidr       = "${var.service_cidr}"
    dns_service_ip     = "${var.dns_service_ip}"
    docker_bridge_cidr = "${var.docker_bridge_cidr}"
  }
 
  tags = {
    ManagedBy = "Terraform"
  }
}
 
resource "azurerm_public_ip" "aks_ingress_egress_ip" {
  name = "aks-ingress-ip"
  location = "${var.location}"
  resource_group_name = "${var.resource_group_name}"
  sku = "Basic"
  allocation_method = "Static"
  domain_name_label = "${var.dns_prefix}"
}
 
resource "azurerm_role_assignment" "public_ip_access_aks_cluster" {
  principal_id         = "${azurerm_azuread_service_principal_password.ad_sp_password.service_principal_id}"
  role_definition_name = "Network Contributor"
  scope                = "${azurerm_public_ip.aks_ingress_egress_ip.id}"
}
 
output "aks_cluster_resource_group" {
  value = "${var.resource_group_name}"
}
 
output "aks_cluster_name" {
  value = "${azurerm_kubernetes_cluster.aks_cluster.name}"
}
 
output "aks_cluster_ingress_ip" {
  value = "${azurerm_public_ip.aks_ingress_egress_ip.ip_address}"
}
 
output "aks_cluster_ingress_fqdn" {
  value = "${azurerm_public_ip.aks_ingress_egress_ip.fqdn}"
}
 
provider "kubernetes" {
  host                   = "${azurerm_kubernetes_cluster.aks_cluster.kube_config_host}"
  client_certificate     = "${base64decode(azurerm_kubernetes_cluster.aks_cluster.kube_config_client_certificate)}"
  client_key             = "${base64decode(azurerm_kubernetes_cluster.aks_cluster.kube_config_client_key)}"
  cluster_ca_certificate = "${base64decode(azurerm_kubernetes_cluster.aks_cluster.kube_config_cluster_ca_certificate)}"
}
